// DirectDraw
// Esempio di Text Flashing
//
// Compilare with NMAKE (Visual C++)

#define NAME "DDExample"
#define TITLE "DirectDraw TextFlash"	// Titolo Applicazione Win

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <ddraw.h>		// DirectDraw
#include <stdlib.h>
#include <stdarg.h>
#include "resource.h"

#define TIMER_ID        1
#define TIMER_RATE      500			// Velocit del flashing di testo
									// 1000 = 1 secondo

LPDIRECTDRAW            lpDD;           // Oggetto DirectDraw
LPDIRECTDRAWSURFACE     lpDDSPrimary;   // DirectDraw FrontBuffer
LPDIRECTDRAWSURFACE     lpDDSBack;      // DirectDraw BackBuffer
BOOL                    bActive;        // controllore attivit applic.


// Stringhe di testo visualizzate
char szFrontMsg[] = "FrontBuffer";
char szBackMsg[] = "BackBuffer";

// Stringa sotto l'effetto FLASH
char szMsg[] = "FLASHING: Testo da leggere - (ESC) per uscire";


/*
 * Chiude gli oggetti in uso da DirectDraw
 */
static void finiObjects( void )
{
    if( lpDD != NULL )
    {
	if( lpDDSPrimary != NULL )
	{
	    lpDDSPrimary->Release();
	    lpDDSPrimary = NULL;
	}
	lpDD->Release();
	lpDD = NULL;
    }
}



// WINDOW_PROC
long FAR PASCAL WindowProc( HWND hWnd, UINT message, 
			    WPARAM wParam, LPARAM lParam )
{
    PAINTSTRUCT ps;
    RECT        rc;
    SIZE        size;
    static BYTE phase = 0;
    HDC         hdc;

    switch( message )
    {
    case WM_ACTIVATEAPP:
	bActive = wParam;
	break;

    case WM_CREATE:
	break;

    case WM_SETCURSOR:
	SetCursor(NULL);
	return TRUE;

    case WM_TIMER:
	// Flip dei due buffer
	// la variabile phase vale
	// 1 = quando  mostrato il FrontBuffer
	// 0 = quando  mostrato il BackBuffer
	if( bActive )
	{
	    if (lpDDSBack->GetDC(&hdc) == DD_OK)
	    {
		SetBkColor( hdc, RGB( 255, 255, 0 ) );	// Sfondo Giallo
		SetTextColor( hdc, RGB( 0, 0, 0 ) );	// Testo Nero
		if( phase )
		{
		    TextOut( hdc, 0, 0, szFrontMsg, lstrlen(szFrontMsg) );
		    phase = 0;
		}
		else
		{
		    TextOut( hdc, 0, 0, szBackMsg, lstrlen(szBackMsg) );
		    phase = 1;
		}
		lpDDSBack->ReleaseDC(hdc);
	    }

	    //ciclo di Page Flipping
		while( 1 )
	    {
		HRESULT ddrval;
		ddrval = lpDDSPrimary->Flip( NULL, 0 );
		if( ddrval == DD_OK )
		{
		    break;
		}
		if( ddrval == DDERR_SURFACELOST )
		{
		    ddrval = lpDDSPrimary->Restore();
		    if( ddrval != DD_OK )
		    {
			break;
		    }
		}
		if( ddrval != DDERR_WASSTILLDRAWING )
		{
		    break;
		}
	    }
	}
	break;
 
    
	//in caso di pressione di ESC terina
	case WM_KEYDOWN:
	switch( wParam )
	{
	case VK_ESCAPE:
	    PostMessage(hWnd, WM_CLOSE, 0, 0);
	    break;
	}
	break;

    //disegna il testo flash al centro dello schermo
	case WM_PAINT:
	BeginPaint( hWnd, &ps );
	GetClientRect(hWnd, &rc);
	GetTextExtentPoint( ps.hdc, szMsg, lstrlen(szMsg), &size );
	SetBkColor( ps.hdc, RGB( 0, 0, 255 ) );		//Sfondo Blu
	SetTextColor( ps.hdc, RGB( 255, 255, 255) );  //Testo Flash in Bianco
	//Posizione al centro dello schermo
	TextOut( ps.hdc, (rc.right - size.cx)/2, (rc.bottom - size.cy)/2,
	    szMsg, sizeof( szMsg )-1 );
	EndPaint( hWnd, &ps );
	break;

    case WM_DESTROY:
	finiObjects();
	PostQuitMessage( 0 );
	break;
    }

    return DefWindowProc(hWnd, message, wParam, lParam);

}


  
  /*************************************************************************
 * doInit - Inizializzazione di una applicazione Windows:
 *          Creazione della finestra, inizializzazione variabili
 *			Restituisce un valore BOOLEAN
 */
static BOOL doInit( HINSTANCE hInstance, int nCmdShow )
{
    HWND                hwnd;
    WNDCLASS            wc;
    DDSURFACEDESC       ddsd;
    DDSCAPS             ddscaps;
    HRESULT             ddrval;
    HDC                 hdc;
    char                buf[256];

    /*
     * SetUp e Registrazione della Classe "window"
     */
    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon( hInstance, IDI_APPLICATION );
    wc.hCursor = LoadCursor( NULL, IDC_ARROW );
    wc.hbrBackground = NULL;
    wc.lpszMenuName = NAME;
    wc.lpszClassName = NAME;
    RegisterClass( &wc );
    
    /*
     * Crea una finestra
     */
    hwnd = CreateWindowEx(
	WS_EX_TOPMOST,
	NAME,
	TITLE,
	WS_POPUP,
	0, 0,
	GetSystemMetrics( SM_CXSCREEN ),
	GetSystemMetrics( SM_CYSCREEN ),
	NULL,
	NULL,
	hInstance,
	NULL );

    if( !hwnd )
    {
	return FALSE;
    }

    ShowWindow( hwnd, nCmdShow );
    UpdateWindow( hwnd );


	/***************
     * Parte relativa a DirectDraw
     *
	 * Crea l'oggetto principale di DirectDraw
     */
    ddrval = DirectDrawCreate( NULL, &lpDD, NULL );
    
	if( ddrval == DD_OK )
    {
	// Prende modo esclusivo a schermo intero
	ddrval = lpDD->SetCooperativeLevel( hwnd,
				DDSCL_EXCLUSIVE | DDSCL_FULLSCREEN );
	if(ddrval == DD_OK )
	{
	    // Setta il modo video su 640x480 8-bit
		ddrval = lpDD->SetDisplayMode( 640, 480, 8 );
	    if( ddrval == DD_OK )
	    {
		// Creazione della Primary Surface con 1 BackBuffer
		ddsd.dwSize = sizeof( ddsd );
		ddsd.dwFlags = DDSD_CAPS | DDSD_BACKBUFFERCOUNT;
		ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE |
				      DDSCAPS_FLIP | 
				      DDSCAPS_COMPLEX;
		ddsd.dwBackBufferCount = 1;
		ddrval = lpDD->CreateSurface( &ddsd, &lpDDSPrimary, NULL );
		if( ddrval == DD_OK )
		{
		    // Localizza il puntatore al BackBuffer
		    ddscaps.dwCaps = DDSCAPS_BACKBUFFER;
		    ddrval = lpDDSPrimary->GetAttachedSurface(&ddscaps, 
							  &lpDDSBack);
		    if( ddrval == DD_OK )
		    {
			
			// Scrive il testo iniziale nel FrontBuffer
			if (lpDDSPrimary->GetDC(&hdc) == DD_OK)
			{
			    //colore di sfondo Giallo
				SetBkColor( hdc, RGB( 255, 255, 0 ) );
			    //colore del testo Nero
				SetTextColor( hdc, RGB( 0, 0, 0 ) );
			    //angolo in alto a sinistra (0,0)
				TextOut( hdc, 0, 0, szFrontMsg, lstrlen(szFrontMsg) );
			    lpDDSPrimary->ReleaseDC(hdc);
			}

			// Scrive il testo iniziale nel BackBuffer
			if (lpDDSBack->GetDC(&hdc) == DD_OK)
			{
			    SetBkColor( hdc, RGB( 255, 255, 0 ) );
			    SetTextColor( hdc, RGB( 0, 0, 0 ) );
			    TextOut( hdc, 0, 0, szBackMsg, lstrlen(szBackMsg) );
			    lpDDSBack->ReleaseDC(hdc);
			}

			// Crea il timer che regola l'animazione "flash"
			if( SetTimer( hwnd, TIMER_ID, TIMER_RATE, NULL ) )
			{
			     return TRUE;
			}
		    }
		}
	    }
	}
    }

    // In caso di errore nella creazione dell'oggetto
	// viene visualizzato un messaggio con il codice di errore
	wsprintf(buf, "Errore Inizializzazione Direct Draw (%08lx)\n", ddrval );
    MessageBox( hwnd, buf, "ERROR", MB_OK );
    finiObjects();	// termina gli oggetti in uso
    DestroyWindow( hwnd );
    return FALSE;
}


/************************************************************************
 * WinMain - Main di un classico programma Win32
 *           Comprende la fase di inizializzazione (chiamata a doInit)
 *           e il ciclo che alterna i messaggi di testo
 */
int PASCAL WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
			LPSTR lpCmdLine, int nCmdShow)
{
    MSG         msg;

    lpCmdLine = lpCmdLine;
    hPrevInstance = hPrevInstance;

    if( !doInit( hInstance, nCmdShow ) )	//Chiamata alla funzione
											//di inizializzazione
    {
	return FALSE;
    }

    while (GetMessage(&msg, NULL, 0, 0))	//Ciclo delle ripetzioni
    {
	TranslateMessage(&msg);
	DispatchMessage(&msg);
    }

    return msg.wParam;

}
